{
THStrings - component to hide strings from view.
The encryption used is a simple encryption not intended to keep out serious
attacks.  It is intended to merely keep the strings hidden from view in the
.EXE file.

At design time or at run time, the strings are not encrypted.  They are only
encrypted when they are stored in the .DFM (and the .EXE) because readdata
and writedata have been provided to handle this automatically.
To see how it works:
- plop one on your form.
- Put in some strings in.
- Save the form.
- Open the form as a .DFM in the IDE and look at the HStr property.
- You will see "garbage". Those are your strings.

example of use:
 (assuming one of the strings in hstrings is password=SomePassword)
 session.addpassword(hstrings1.strings.values['password']);

Why named HStrings?? because the class name shows up in the .EXE file.
If I called it something like THiddenStrings or something it would make it
too easy.

Someone could add a real encryption algorithm and make this a little more
useful.

Georg Zimmer <gzimmer@usa.net>
 }

unit HidStr;

interface

uses
  SysUtils, Classes;

type
  THStrings = class(TComponent)
  private
    fStrings: tstringlist;
  protected
    function Encrypt(const str: string): string; virtual;
    function Decrypt(const str: string): string; virtual;
  public
    procedure SetStrings(value: tstringlist);
    procedure Readdata(reader:treader);
    procedure Writedata(writer:twriter);
    procedure DefineProperties(filer: tfiler);override;
    constructor Create(aowner: tcomponent);override;
    destructor Destroy; override;
  published
    property Strings: TStringlist read fStrings write SetStrings stored false;
  end;

procedure Register;

implementation

{$IFDEF WIN32}
  {$R HidStr.d32} { 32-bit-HidStr.dcr is renamed to d32 }
{$ELSE DEF WIN32}
  {$R HidStr.d16}
{$ENDIF DEF WIN32}


const
  mask = 'EV7V5G'#39'v7'#20#39'VFFV';
{ tHStrings en-/decrypts by doing a XOR with mask.
  Define your own mask and make it long.
  Don't change it if you are using a tHStrings-component on a form.
  To gain more security, overwrite Ecrypt and Decrypt.  }

function tHStrings.Decrypt(const str: string): string;
var n: integer;
    t: word;
    l: word;
begin {encrypt (or decrypt!) by xor }
   result:='';
   for n:=1 to length(str) do
      result:=result+
        chr(ord(str[n]) xor ord(mask[((n-1) mod length(mask)) +1]));
end;

function tHStrings.Encrypt(const str: string): string;
begin
   result:=Decrypt(str); { symmetrical encryption }
end;

procedure tHStrings.SetStrings(value: tstringlist);
begin
   fStrings.assign(value);
end;

procedure THStrings.DefineProperties(filer: tfiler);
begin
   inherited defineproperties(filer);
   filer.DefineProperty('HStr',Readdata,WriteData,(fStrings.count>0));
end;

constructor THStrings.Create(aowner: tcomponent);
begin
   inherited create(aowner);
   fStrings:=tstringlist.create;
end;

destructor tHStrings.Destroy;
begin
   fStrings.free;
   inherited destroy;
end;

procedure THStrings.Readdata(reader:treader);
var n: integer;
begin
   reader.readlistbegin;
   fStrings.clear;
   while not reader.endoflist do begin
      fStrings.add(Decrypt(reader.readstring));
   end;
   reader.readlistend;
end;

procedure THStrings.Writedata(writer:twriter);
var n: integer;
begin
   writer.writelistbegin;
   for n:=0 to fStrings.count-1 do begin
      writer.writestring(Encrypt(fStrings[n]));
   end;
   writer.writelistend;
end;

procedure Register;
begin
  RegisterComponents('Samples', [THStrings]);
end;

end.

